using System;
using System.Data;
using ByteFX.Data.MySqlClient;
using System.Collections.Specialized;
using System.Xml;

namespace BusObjCS {
	/// <remarks> The PubsAccess class is the fundamental class for accessing information about authors in the system. </remarks>
	public class PubsAccess{
		private string m_connStr;
		
		/// <summary> Initializes a new instance of the PubsAccess class </summary>
		public PubsAccess () {}

		/// <summary> An overloaded form of the PubAccess class constructor that takes a database connection string as an argument. </summary>
		/// <param name="connStr"> Connection string for the database. </param>
		public PubsAccess (string connStr){
			m_connStr = connStr;
		}
		
		/// <summary> Gets or sets the database connection string for the methods in this class that require database access. </summary>
		public string DatabaseConnection{
			set {m_connStr = value;}
			get {return m_connStr;}
		}
		
		/// <summary> Returns a DataSet with information on all of the authors in the system. </summary>
		/// <returns> A DataSet object populated with the names and IDs of authors in the system. </returns>
		/// <example>
		/// <code>
		/// BusObjCS.PubsAccess objPubs = new BusObjCS.PubsAccess();
		/// String dsn = "Data Source=localhost; Database=pubs; User Id=root; Password=";
		/// objPubs.DatabaseConnection = dsn;
		/// // Invoke the GetAuthors method
		/// DataSet authors = objPubs.GetAuthors();
		/// </code>
		/// </example>
		public DataSet GetAuthors () {
			MySqlConnection dbConnection = new MySqlConnection(m_connStr);
			DataSet authors = new DataSet();
			try{
				dbConnection.Open();
				MySqlDataAdapter sqlCommand = new MySqlDataAdapter("SELECT au_id, concat(au_fname,' ',au_lname) AS au_name FROM authors", dbConnection);
				sqlCommand.Fill(authors, "authors");
			}
			catch (MySqlException ex){
				throw(ex);
			}
			finally{
				dbConnection.Close();
			}
			return authors;
		}
	
		/// <summary> Returns all of the titles for a given author ID. </summary>
		/// <param name="authorID"> The ID of the author for which the titles should be retrieved. </param>
		/// <returns> A SringCollection populated with the tities for the AuthorID parameter submitted. </returns>
		/// <example>
		/// <code>
		/// BusObjCS.PubsAccess objPubs = new BusObjCS.PubsAccess();
		/// String dsn = "Data Source=localhost; Database=pubs; User Id=root; Password=";
		/// objPubs.DatabaseConnection = dsn;
		/// // Invoke the GetTitlesForAuthor method
		/// StringCollection titles = objPubs.GetTitlesForAuthor(Request.QueryString["auid"]);
		/// </code>
		/// </example>
		public StringCollection GetTitlesForAuthor (string authorID){
			StringCollection collTitles = new StringCollection();
			MySqlConnection dbConnection = new MySqlConnection(m_connStr);
			DataSet titles = new DataSet();
			try{
				dbConnection.Open();
				MySqlDataAdapter sqlCommand = new MySqlDataAdapter("SELECT title FROM titles INNER JOIN titleauthor ON titles.title_id = titleauthor.title_id WHERE (au_id = '" + authorID + "')", dbConnection);
				sqlCommand.Fill(titles, "titles");
			}
			catch (MySqlException ex){
				throw(ex);
			}
			finally{
				dbConnection.Close();
			}
			DataRow[] titleRows;
			titleRows = titles.Tables["titles"].Select();
			for (int i=0; i <= titleRows.Length - 1; i++){
				collTitles.Add (titleRows[i]["title"].ToString());
			}	
			return collTitles;	
		}
		
		/// <summary> Calculates the total books sales for a given author ID.</summary>
		/// <param name="authorID"> The ID of the author for whom the total sales should be retrieved. </param>
		/// <returns> An integer which represents the total sales for the AuthorID submitted or 0 if no sales are recorded. </returns>
		/// <example>
		/// <code>
		/// BusObjCS.PubsAccess objPubs = new BusObjCS.PubsAccess();
		/// String dsn = "Data Source=localhost; Database=pubs; User Id=root; Password=";
		/// objPubs.DatabaseConnection = dsn;
		/// // Invoke the TotalSalesForAuthor method
		/// totalSales.Text = String.Format("{0:c}", objPubs.TotalSalesForAuthor(Request.QueryString["auid"]));
		/// </code>
		/// </example>
		public int TotalSalesForAuthor (string authorID){
			MySqlConnection dbConnection = new MySqlConnection(m_connStr);
			DataSet SalesTotal = new DataSet();
			try{
				dbConnection.Open();
				MySqlDataAdapter sqlCommand = new MySqlDataAdapter("SELECT SUM(ytd_sales) AS TotalSales FROM titles INNER JOIN titleauthor ON titles.title_ID = titleauthor.title_id WHERE (au_id = '" + authorID + "')", dbConnection);
				sqlCommand.Fill(SalesTotal, "SalesTotal");
			}
			catch (MySqlException ex){
				throw(ex);
			}
			finally{
				dbConnection.Close();
			}
			
			DataTable titleTable;
			titleTable = SalesTotal.Tables["SalesTotal"];
			if (!(titleTable.Rows[0].IsNull("TotalSales"))){
				return Convert.ToInt32(titleTable.Rows[0]["TotalSales"]);	
			}
			else
				return 0;
		}
	}
}
